﻿
using System.Collections;

public static class Program
{
    // https://github.com/dotnet/csharplang/blob/main/proposals/list-patterns.md
    // https://devblogs.microsoft.com/dotnet/early-peek-at-csharp-11-features/#c-11-preview-list-patterns

    public static void Main()
    {
        {
            int[] arr1 = { 1, 2, 10 };
            int[] arr2 = { 1, 2, 5, 10 };
            int[] arr3 = { 1, 2, 5, 6, 7, 8, 9, 10 };

            int[] arr4 = { 1, 3 };
            int[] arr5 = { 1, 3, 6 };
            int[] arr6 = { 2, 3 };

            Console.WriteLine(CheckSwitch(arr1)); // 1
            Console.WriteLine(CheckSwitch(arr2)); // 1
            Console.WriteLine(CheckSwitch(arr3)); // 1

            Console.WriteLine(CheckSwitch(arr4)); // 2
            if (arr5 is [1, ..])
            {
                Console.WriteLine(3); // 3
            }
            Console.WriteLine(CheckSwitch(arr6)); // 50
        }

        Console.WriteLine();

        {
            List<int> list1 = new List<int>(new[] { 1, 3 });
            Console.WriteLine(CheckSwitch(list1)); // 3
        }

        Console.WriteLine();

        {
            IList<int> list1 = new List<int>(new[] { 1, 1, 2, 3 });
            Console.WriteLine(CheckSwitch(list1)); // 1

            IList<int> list2 = new List<int>(new[] { 5, 6, 1, 9 });
            Console.WriteLine(CheckSwitch(list2)); // 2
        }

        Console.WriteLine();

        {
            ArrayList arr1 = new ArrayList(new[] { 1, 3, 5 });
            Console.WriteLine(CheckSwitch(arr1)); // 1

            ArrayList arr2 = new ArrayList(new[] { 1, 10, 5 });
            Console.WriteLine(CheckSwitch(arr2)); // 1

            List<int[]> list = new List<int[]>(new[] { new[] { 1, 10, 5 } });
            Console.WriteLine(CheckSwitch(list)); // 1
        }

        Console.WriteLine();

        {
            NaturalNumber list = new NaturalNumber();
            if (list is [1, .., Int32.MaxValue])
            {
                Console.WriteLine(true); // True
            }
        }

        {
            NaturalNumber list = new NaturalNumber();

            if (list is [1, .. var middle, Int32.MaxValue])
            {
                foreach (var elem in middle.Take(10))
                {
                    Console.Write($"{elem},");
                }
            }
        }
    }

    static int CheckSwitch(ArrayList values)
        => values switch
        {
            [_, >= 3, ..] or [_, <= 10, ..] => 1,
            [..] => -1,
        };

    static int CheckSwitch(IList<int> values)
    {
        const int condition = 1;

        switch (values)
        {
            //case [_, .., 1]: return 1;
            //case [.., _, 1]: return 2;

            case [_, 1, ..]: return 1;
            case [.., condition, _]: return 2;
        }

        return 0;
    }

    static int CheckSwitch(int[] values)
        => values switch
        {
            [1, 2, .., 10] => 1,
            [1, _] => 2,
            [1, ..] => 3,
            [..] => 50
        };

    static int CheckSwitch(List<int> values)
        => values switch
        {
            [1, 2] => 1,
            [1, _] => 2,
            [..] => 3,
        };

    static int CheckSwitch(List<int[]> values)
        => values switch
        {
            [ [1, .., 5]] => 1,
            [..] => -1,
        };
}

public class NaturalNumber
{
    int _length = Int32.MaxValue;

    public int this[int index]
    {
        get { return index + 1; }
    }

    public IEnumerable<int> this[System.Range range]
    {
        get
        {
            int startPos = range.Start.Value + 1;
            int endPos = _length - range.End.Value - startPos + 1;

            if (range.End.IsFromEnd == false)
            {
                endPos = range.End.Value - startPos + 1;
            }

            return Enumerable.Range(startPos, endPos);
        }

        set { }
    }

    public int Length => _length;

    // 또는,
    // public int Count => Int32.MaxValue;
}